#include "error.h"
#include <stdio.h>
#include <stdlib.h>
#include <assert.h>

GUID Error::guid_ = { 0 };
HRESULT Error::hResult_ = 0;
DWORD Error::dwHelpContext_ = 0;
char Error::szDescription_[ MAX_STRING_LEN] = { 0 };
char Error::szHelpFile_[ MAX_STRING_LEN] = { 0 };
char Error::szHelpSource_[ MAX_STRING_LEN] = { 0 };
char Error::szHResultText_[ MAX_STRING_LEN] = { 0 };

void 
Error::clearError()
{
    hResult_ = S_OK;
    dwHelpContext_ = 0;
    *szDescription_ = 0;
    *szHelpFile_ = 0;
    *szHelpSource_ = 0;
    *szHResultText_ = 0;
}

bool
Error::setError( const SupportsErrorInfo &object, HRESULT hResult) 
{
    // Clear previous error, if any
    clearError();

    // Check object for error info (if no object, no error)
    IUnknown *pUnk;
    IID *riid;
    object.getErrorInfoParams( pUnk, riid);
    if (!pUnk)
        return true;

    // Get IErrorInfo
    hResult_ = hResult;
    ISupportErrorInfo *pSupportErrorInfo = 0;
    IErrorInfo *pErrorInfo = 0;
    if (pUnk && SUCCEEDED(pUnk->QueryInterface(IID_ISupportErrorInfo, (void**)&pSupportErrorInfo)))
    {
        if (SUCCEEDED(pSupportErrorInfo->InterfaceSupportsErrorInfo( *riid)))
            ::GetErrorInfo(0, &pErrorInfo);
        pSupportErrorInfo->Release();
    }

    if (!pErrorInfo)
        return false;

    BSTR bstrDescription;
    pErrorInfo->GetDescription( &bstrDescription);
    if (SysStringLen( bstrDescription) > 0)
        wcstombs( szDescription_, bstrDescription, MAX_STRING_LEN);
	SysFreeString( bstrDescription);

    pErrorInfo->GetGUID( &guid_);
    pErrorInfo->GetHelpContext( &dwHelpContext_);

    BSTR bstrHelpFile;
    pErrorInfo->GetHelpFile( &bstrHelpFile);
    if (SysStringLen( bstrHelpFile) > 0)
        wcstombs( szHelpFile_, bstrHelpFile, MAX_STRING_LEN);
	SysFreeString( bstrHelpFile);

    BSTR bstrSource;
    pErrorInfo->GetSource( &bstrSource);
    if (SysStringLen( bstrSource) > 0)
        wcstombs( szHelpSource_, bstrSource, MAX_STRING_LEN);
	SysFreeString( bstrSource);

    if (hResult_ != S_OK)
        ::FormatMessage( FORMAT_MESSAGE_FROM_SYSTEM | FORMAT_MESSAGE_IGNORE_INSERTS, 
                0,
                hResult_, 
                MAKELANGID( LANG_NEUTRAL, SUBLANG_DEFAULT), 
                szHResultText_, MAX_STRING_LEN, 0);

    pErrorInfo->Release();

    return true;
}


