#include "stdafx.h"

#include "reply.h"
#include "resultSet.h"

#include <assert.h>
#include "error.h"

#if !defined( HDNONAMESPACE)
namespace HdCPPWrapper {
#endif

Reply::Reply(IHdReply *pReply)
    : pReply_(pReply), pReplyInfo_( 0)
{
    if (pReply_)
        pReply_->AddRef();
}

Reply::Reply(const Reply& rhs)
    : pReply_( 0), pReplyInfo_( 0)
{
    *this = rhs;
}

Reply::~Reply()
{
    if (pReply_)
        pReply_->Release();
    if (pReplyInfo_)
        pReplyInfo_->Release();
}

Reply&
Reply::operator=(const Reply& rhs)
{
    setReply( rhs.pReply_);

    return *this;
}

void 
Reply::getErrorInfoParams( IUnknown *&pUnk, const IID *&riid) const
{ 
    pUnk = pReply_; 
    riid = &IID_IHdReply; 
}


ResultSet
Reply::operator[](unsigned long index)
{
    assert(index >= 1);

    VARIANT resultSetVariant;
    ::VariantInit( &resultSetVariant);

    if (pReply_)
    {
        HRESULT hr = pReply_->get_Item(index, &resultSetVariant);
        if (FAILED( hr))
            Error::setError( *this, hr);
    }
    else
	    assert(pReply_);

    ResultSet rs( resultSetVariant);
    ::VariantClear(&resultSetVariant);

    return rs;
}

void
Reply::setReply(IHdReply *pReply)
{
    if (pReply != pReply_)
    {
        if (pReply_)
        {
            pReply_->Release();
            pReply_ = 0;
        }
        if (pReply_ = pReply)
            pReply_->AddRef();
    }

    if (pReplyInfo_)
        pReplyInfo_->Release();

    pReply_->QueryInterface( IID_IHdReplyInfo, (void**)&pReplyInfo_);
}

long
Reply::Count() const
{
    long nCount = -1;
    if (pReply_)
        pReply_->get_Count(&nCount);

    return nCount;
}

Reply::Status Reply::getStatus() const
{
    if (!pReply_)
    {
        assert( pReply_);
        return Reply::REDEEMABLE;
    }

    HD_REPLY_STATUS status;
    const HRESULT hr = pReply_->get_Status( &status);
    if (!SUCCEEDED( hr))
    {
        assert( SUCCEEDED( hr));
        return Reply::REDEEMABLE;
    }

    switch (status)
    {	
    case REDEEMABLE:
        return Reply::REDEEMABLE;
    case PROCESSING:
        return Reply::PROCESSING;
    case ABORTED:
        return Reply::ABORTED;
    default:
        assert( false);
        return Reply::REDEEMABLE;
    }
}

VARIANT 
Reply::getReplyInfo( const BSTR &bsName) const
{
    // clear the global error object
    Error::clearError();

    VARIANT value;
    VariantInit( &value);

    if (!pReplyInfo_)
    {
        assert(pReplyInfo_);
        return value;
    }

    HRESULT hr = pReplyInfo_->get_Value( bsName, &value);
    if (FAILED(hr))
        Error::setError( *this, hr);

    return value;
}

bool
Reply::getReplyInfo( int nIndex, BSTR *pName) const
{
    // clear the global error object
    Error::clearError();

    // Validate parameters
    if (!pReplyInfo_ || !pName)
    {
        assert( pReplyInfo_);
        assert( pName);
        return false;
    }

    // Get the enumerator interface
    IUnknown *pUnk = 0;
    HRESULT hr = pReplyInfo_->get__NewEnum( &pUnk);
    if (FAILED( hr))
    {
        Error::setError( *this, hr);
        return false;
    }
    IEnumVARIANT *pEnum = 0;
    hr = pUnk->QueryInterface( IID_IEnumVARIANT, (void **) &pEnum);
    if (FAILED( hr))
    {
        Error::setError( *this, hr);
        return false;
    }

    // Skip to the desired index and retrieve the name; note the 1-based index
    VARIANT name;
    ::VariantInit( &name);
    unsigned long retrieved = 0;
    if (nIndex > 1)
        pEnum->Skip( nIndex - 1);
    pEnum->Next( 1, &name, &retrieved);
    const bool bSuccess = retrieved == 1 && name.vt == VT_BSTR;

    // Extract the name
    if (bSuccess)
        *pName = name.bstrVal;

    pEnum->Release();
    return bSuccess;
}

#if !defined( HDNONAMESPACE)
}
#endif


