#include "mutex.h"
#include "event.h"

template<class T>
class ResultIOU_
{
public:
    ResultIOU_();
    ~ResultIOU_();

    void Abort();
    T Redeem();
    bool Redeemable() const;
    bool Aborted() const;
    void setResult(T t);

private:
    T t_;
	Event closedEvent_;
    bool resultReady_;
    bool aborted_;
};

template<class T>
inline
ResultIOU_<T>::ResultIOU_()
: resultReady_(false), aborted_(false)
{
}

template<class T>
inline
ResultIOU_<T>::~ResultIOU_()
{
}

template<class T>
inline
void ResultIOU_<T>::Abort()
{
	Mutex mutex;

    // set result ready flag
    resultReady_ = true;
    
    // set aborted flag
    aborted_ = true;

    // Release any threads blocked in Redeem().
    mutex.Release();
}

template<class T>
inline
T ResultIOU_<T>::Redeem()
{
	Mutex mutex;

    while (!resultReady_)
		closedEvent_.Wait();

    return t_;
}

template<class T>
inline
bool ResultIOU_<T>::Redeemable() const
{
	Mutex mutex;

    return resultReady_;
}


template<class T>
inline
bool ResultIOU_<T>::Aborted() const
{
	Mutex mutex;

    return aborted_;
}

template<class T>
inline
void ResultIOU_<T>::setResult(T t)
{
	Mutex mutex;

    if (aborted_)
        return;

    t_ = t;
    resultReady_ = true;

    // Signal threads blocked in _redeem().
	closedEvent_.Signal();

    mutex.Release();
}
